package com.example.nto.service.impl;

import com.example.nto.dto.CodeDTO;
import com.example.nto.dto.EmployeeDTO;
import com.example.nto.dto.RegisterDTO;
import com.example.nto.entity.Authority;
import com.example.nto.entity.Code;
import com.example.nto.entity.Employee;
import com.example.nto.exceptions.AuthorityNotFoundException;
import com.example.nto.exceptions.CodeNotFoundException;
import com.example.nto.exceptions.EmployeeAlreadyExistException;
import com.example.nto.exceptions.EmployeeNotFoundException;
import com.example.nto.repository.AuthorityRepository;
import com.example.nto.repository.CodeRepository;
import com.example.nto.repository.EmployeeRepository;
import com.example.nto.service.EmployeeService;
import com.example.nto.util.EmployeeMapper;
import lombok.AllArgsConstructor;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Optional;
import java.util.Set;

@Service
@AllArgsConstructor
public class EmployeeServiceImpl implements EmployeeService {

    private EmployeeRepository employeeRepository;
    private final PasswordEncoder passwordEncoder;
    private final AuthorityRepository authorityRepository;
    private final CodeRepository codeRepository;

    @Override
    public void updateLocalTime(Employee employee) {
        employee.setLastVisit(LocalDateTime.now().toString());
        employeeRepository.save(employee);
    }

    @Override
    public Employee findEmployeeByUsername(String login) {

        return employeeRepository.findEmployeeByUsername(login).orElseThrow(() -> new EmployeeNotFoundException("Employee with username "+login+" not found"));
    }

    public EmployeeDTO findEmployeeDTOByUsername(String login) {

        return EmployeeMapper.convertToDTO(employeeRepository.findEmployeeByUsername(login).orElseThrow(() -> new EmployeeNotFoundException("Employee with username "+login+" not found")));
    }

    @Override
    public EmployeeDTO createEmployee(RegisterDTO dto) {

        Optional<Employee> optionalEmployee = employeeRepository.findEmployeeByUsername(dto.getUsername());
        if(optionalEmployee.isPresent()){
            throw new EmployeeAlreadyExistException("Employee with username " + dto.getUsername() +" already exist");
        }

        Optional<Authority> optionalAuthority = authorityRepository.findByAuthority("ROLE_EMPLOYEE");
        if(optionalAuthority.isEmpty()){
            throw new AuthorityNotFoundException("Authority not found");
        }

        Employee employee = new Employee();
        employee.setName(dto.getName());
        employee.setUsername(dto.getUsername());

        // Encoding password
        employee.setPassword(passwordEncoder.encode(dto.getPassword()));

        employee.setPhoto(dto.getPhoto());
        employee.setJobPos(dto.getJobPos());
        employee.setLastVisit(LocalDateTime.now().toString());

        employee.setAuthorities(Set.of(optionalAuthority.get()));

        return EmployeeMapper.convertToDTO(employeeRepository.save(employee));
    }

    @Override
    public EmployeeDTO findEmployeeByCode(CodeDTO dto) {
        Optional<Code> optionalCode = codeRepository.findByValue(dto.getValue());
        if(optionalCode.isEmpty()){
            throw new CodeNotFoundException("Qr-Code "+dto.getValue()+" not found");
        }

        Optional<Employee> optionalEmployee = employeeRepository.findEmployeeByCode(optionalCode.get());
        if(optionalEmployee.isEmpty()){
            throw new EmployeeNotFoundException("Employee by qr-code "+dto.getValue()+" not found");
        }

        return EmployeeMapper.convertToDTO(optionalEmployee.get());
    }


}

